@@ -1,4 +1,4 @@
-// Copyright (C) 2021  Tassilo Horn <tsdh@gnu.org>
+// Copyright (C) 2021-2022  Tassilo Horn <tsdh@gnu.org>
 //
 // This program is free software: you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the Free
@@ -16,6 +16,7 @@
 //! Convenience data structures built from the IPC structs.
 
 use crate::config;
+use crate::rtfmt;
 use crate::util;
 use crate::util::DisplayFormat;
 use lazy_static::lazy_static;
@@ -427,15 +428,19 @@ pub fn get_tree<'a>(
 lazy_static! {
     static ref APP_NAME_AND_VERSION_RX: regex::Regex =
         regex::Regex::new("(.+)(-[0-9.]+)").unwrap();
+    static ref PLACEHOLDER_RX: regex::Regex = regex::Regex::new(
+        r"\{(?P<name>[^}:]+)(?::(?P<fmtstr>\{[^}]*\})(?P<clipstr>[^}]*))?\}"
+    )
+    .unwrap();
 }
 
-fn maybe_html_escape(do_it: bool, text: &str) -> String {
+fn maybe_html_escape(do_it: bool, text: String) -> String {
     if do_it {
-        text.replace("<", "&lt;")
-            .replace(">", "&gt;")
-            .replace("&", "&amp;")
+        text.replace('<', "&lt;")
+            .replace('>', "&gt;")
+            .replace('&', "&amp;")
     } else {
-        text.to_string()
+        text
     }
 }
 
@@ -470,16 +475,8 @@ impl DisplayFormat for DisplayNode<'_> {
             Type::Container => cfg.get_format_container_format(),
             Type::Window => cfg.get_format_window_format(),
         };
-        fmt.replace("{indent}", &indent.repeat(self.get_indent_level()))
-            .replace("{layout}", format!("{:?}", self.node.layout).as_str())
-            .replace("{id}", format!("{}", self.node.id).as_str())
-            .replace(
-                "{marks}",
-                &maybe_html_escape(
-                    html_escape,
-                    &format_marks(&self.node.marks),
-                ),
-            )
+        let fmt = fmt
+            .replace("{indent}", &indent.repeat(self.get_indent_level()))
             .replace(
                 "{urgency_start}",
                 if self.node.urgent {
@@ -496,28 +493,6 @@ impl DisplayFormat for DisplayNode<'_> {
                     ""
                 },
             )
-            .replace(
-                "{app_name}",
-                &maybe_html_escape(html_escape, self.node.get_app_name()),
-            )
-            .replace(
-                "{output_name}",
-                &maybe_html_escape(
-                    html_escape,
-                    self.tree
-                        .get_parent_node_of_type(self.node.id, Type::Output)
-                        .map_or("<no_output>", |w| w.get_name()),
-                ),
-            )
-            .replace(
-                "{workspace_name}",
-                &maybe_html_escape(
-                    html_escape,
-                    self.tree
-                        .get_parent_node_of_type(self.node.id, Type::Workspace)
-                        .map_or("<no_workspace>", |w| w.get_name()),
-                ),
-            )
             .replace(
                 "{app_icon}",
                 util::get_icon(self.node.get_app_name(), &icon_dirs)
@@ -534,15 +509,38 @@ impl DisplayFormat for DisplayNode<'_> {
                     .map(|i| i.to_string_lossy().into_owned())
                     .unwrap_or_else(String::new)
                     .as_str(),
-            )
-            .replace(
-                "{title}",
-                &maybe_html_escape(html_escape, self.node.get_name()),
-            )
-            .replace(
-                "{name}",
-                &maybe_html_escape(html_escape, self.node.get_name()),
-            )
+            );
+
+        PLACEHOLDER_RX
+            .replace_all(&fmt, |caps: &regex::Captures| {
+                let value = match &caps["name"] {
+                    "id" => self.node.id.to_string(),
+                    "app_name" => self.node.get_app_name().to_string(),
+                    "layout" => format!("{:?}", self.node.layout),
+                    "name" | "title" => self.node.get_name().to_string(),
+                    "output_name" => self
+                        .tree
+                        .get_parent_node_of_type(self.node.id, Type::Output)
+                        .map_or("<no_output>", |w| w.get_name())
+                        .to_string(),
+                    "workspace_name" => self
+                        .tree
+                        .get_parent_node_of_type(self.node.id, Type::Workspace)
+                        .map_or("<no_workspace>", |w| w.get_name())
+                        .to_string(),
+                    "marks" => format_marks(&self.node.marks),
+                    _ => caps[0].to_string(),
+                };
+                let fmt_str = caps.name("fmtstr").map_or("{}", |m| m.as_str());
+                let clipped_str =
+                    caps.name("clipstr").map_or("", |m| m.as_str());
+
+                maybe_html_escape(
+                    html_escape,
+                    rtfmt::format(fmt_str, &value, clipped_str),
+                )
+            })
+            .into()
     }
 
     fn get_indent_level(&self) -> usize {
@@ -571,3 +569,26 @@ impl DisplayFormat for DisplayNode<'_> {
         }
     }
 }
+
+#[test]
+fn test_placeholder_rx() {
+    let caps = PLACEHOLDER_RX.captures("Hello, {place}!").unwrap();
+    assert_eq!(caps.name("name").unwrap().as_str(), "place");
+    assert_eq!(caps.name("fmtstr"), None);
+    assert_eq!(caps.name("clipstr"), None);
+
+    let caps = PLACEHOLDER_RX.captures("Hi, {place:{:>10.10}}!").unwrap();
+    assert_eq!(caps.name("name").unwrap().as_str(), "place");
+    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:>10.10}");
+    assert_eq!(caps.name("clipstr").unwrap().as_str(), "");
+
+    let caps = PLACEHOLDER_RX.captures("Hello, {place:{:.5}…}!").unwrap();
+    assert_eq!(caps.name("name").unwrap().as_str(), "place");
+    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:.5}");
+    assert_eq!(caps.name("clipstr").unwrap().as_str(), "…");
+
+    let caps = PLACEHOLDER_RX.captures("Hello, {place:{:.5}...}!").unwrap();
+    assert_eq!(caps.name("name").unwrap().as_str(), "place");
+    assert_eq!(caps.name("fmtstr").unwrap().as_str(), "{:.5}");
+    assert_eq!(caps.name("clipstr").unwrap().as_str(), "...");
+}
